/**
 * Defines base types used throughout the API.
 *
 * @author Ivano Bilenchi
 *
 * @file
 */

#ifndef TME_BASE_TYPES_H
#define TME_BASE_TYPES_H

#include "tme_compat.h"
#include "uhash.h"
#include "vector.h"

#include <inttypes.h>

TME_BEGIN_DECLS

/**
 * Unsigned integer type.
 *
 * @typedef tme_uint_t
 */

/**
 * Maximum value of a tme_uint_t variable.
 *
 * @def TME_UINT_MAX
 */

/**
 * Format string for tme_uint_t variables.
 *
 * @def TME_UINT_FMT
 */

#if defined TME_TINY
    typedef uint16_t tme_uint_t;
    #define TME_UINT_MAX UINT16_MAX
    #define TME_UINT_FMT PRIu16
#elif defined TME_HUGE
    typedef uint64_t tme_uint_t;
    #define TME_UINT_MAX UINT64_MAX
    #define TME_UINT_FMT PRIu64
#else
    typedef uint32_t tme_uint_t;
    #define TME_UINT_MAX UINT32_MAX
    #define TME_UINT_FMT PRIu32
#endif

TME_END_DECLS

#endif // TME_BASE_TYPES_H
