/**
 * Declares convenience APIs for working with reasoner instances backed by Cowl ontologies.
 *
 * @author Ivano Bilenchi
 *
 * @file
 */

#ifndef TME_COWL_REASONER_H
#define TME_COWL_REASONER_H

#include "cowl_std.h"
#include "tme_std.h"
#include "tme_abox_entry_type.h"

TME_BEGIN_DECLS

/// @cond
cowl_struct_decl(CowlOntology);
tme_struct_decl(TmeReasoner);
/// @endcond

/**
 * Allocates a new reasoner instance backed by the specified Cowl ontology.
 *
 * @param ontology Ontology instance.
 * @return Reasoner instance.
 */
TmeReasoner* tme_reasoner_alloc_ontology(CowlOntology *ontology);

/**
 * Loads individuals from the specified ontology into the reasoner.
 *
 * @param reasoner Reasoner instance.
 * @param ontology Ontology instance.
 * @param type Type of the loaded individuals.
 */
void tme_reasoner_load_individuals(TmeReasoner *reasoner, CowlOntology *ontology,
                                   TmeABoxEntryType type);

TME_END_DECLS

#endif // TME_COWL_REASONER_H
