/**
 * Declares the interface of the Cowl string provider.
 *
 * @author Ivano Bilenchi
 *
 * @file
 */

#ifndef TME_COWL_STRING_PROVIDER_H
#define TME_COWL_STRING_PROVIDER_H

#include "cowl_std.h"
#include "tme_std.h"

TME_BEGIN_DECLS

/// @cond
cowl_struct_decl(CowlOntology);
tme_struct_decl(TmeStringProvider);
/// @endcond

/**
 * Allocates a new string provider backed by a Cowl ontology.
 *
 * @param ontology Ontology instance.
 * @return String provider instance.
 */
TmeStringProvider* tme_cowl_string_provider_alloc(CowlOntology *ontology);

/**
 * Deallocates the string provider.
 *
 * @param provider String provider instance.
 */
void tme_cowl_string_provider_free(TmeStringProvider *provider);

/**
 * Loads strings from the specified Cowl ontology.
 *
 * @param provider String provider instance.
 * @param ontology Ontology instance.
 *
 * @note This function is automatically called on the ontology referenced when allocating
 *       the string provider. It needs to be called manually if multiple ontologies are
 *       used to load knowledge into the reasoner.
 */
void tme_cowl_string_provider_load_strings(TmeStringProvider *provider, CowlOntology *ontology);

TME_END_DECLS

#endif // TME_COWL_STRING_PROVIDER_H
